@@ -3,14 +3,16 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2007, 2014, 2017 Lukáš Lalinský
-# Copyright (C) 2008, 2014, 2019-2021 Philipp Wolfer
+# Copyright (C) 2008, 2014, 2019-2023 Philipp Wolfer
 # Copyright (C) 2012, 2017 Wieland Hoffmann
 # Copyright (C) 2012-2014 Michael Wiencek
-# Copyright (C) 2013-2016, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2016, 2018-2022 Laurent Monin
 # Copyright (C) 2016 Suhas
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2017 Sophist-UK
 # Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2020-2021 Gabriel Ferreira
+# Copyright (C) 2021-2022 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,6 +29,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
+from collections import defaultdict
+import inspect
 from operator import itemgetter
 import os
 import shutil
@@ -42,9 +46,16 @@ from picard import (
     PICARD_VERSION,
     log,
 )
+from picard.profile import UserProfileGroups
 from picard.version import Version
 
 
+class Memovar:
+    def __init__(self):
+        self.dirty = True
+        self.value = None
+
+
 class ConfigUpgradeError(Exception):
     pass
 
@@ -58,16 +69,11 @@ class ConfigSection(QtCore.QObject):
         self.__qt_config = config
         self.__name = name
         self.__prefix = self.__name + '/'
-        self.__prefix_len = len(self.__prefix)
+        self._memoization = defaultdict(Memovar)
 
     def key(self, name):
         return self.__prefix + name
 
-    def _subkeys(self):
-        for key in self.__qt_config.allKeys():
-            if key[:self.__prefix_len] == self.__prefix:
-                yield key[self.__prefix_len:]
-
     def __getitem__(self, name):
         opt = Option.get(self.__name, name)
         if opt is None:
@@ -77,15 +83,27 @@ class ConfigSection(QtCore.QObject):
     def __setitem__(self, name, value):
         key = self.key(name)
         self.__qt_config.setValue(key, value)
+        self._memoization[key].dirty = True
 
     def __contains__(self, name):
         return self.__qt_config.contains(self.key(name))
 
+    def as_dict(self):
+        return {
+            key: self[key] for section, key
+            in list(Option.registry)
+            if section == self.__name
+        }
+
     def remove(self, name):
         key = self.key(name)
         config = self.__qt_config
         if config.contains(key):
             config.remove(key)
+        try:
+            del self._memoization[key]
+        except KeyError:
+            pass
 
     def raw_value(self, name, qtype=None):
         """Return an option value without any type conversion."""
@@ -99,16 +117,108 @@ class ConfigSection(QtCore.QObject):
     def value(self, name, option_type, default=None):
         """Return an option value converted to the given Option type."""
         if name in self:
-            try:
-                value = self.raw_value(name, qtype=option_type.qtype)
-                value = option_type.convert(value)
-            except Exception as why:
-                log.error('Cannot read %s value: %s', self.key(name), why, exc_info=True)
-                value = default
-            return value
+            key = self.key(name)
+            memovar = self._memoization[key]
+
+            if memovar.dirty:
+                try:
+                    value = self.raw_value(name, qtype=option_type.qtype)
+                    value = option_type.convert(value)
+                    memovar.dirty = False
+                    memovar.value = value
+                except Exception as why:
+                    log.error('Cannot read %s value: %s', self.key(name), why, exc_info=True)
+                    value = default
+                return value
+            else:
+                return memovar.value
         return default
 
 
+class SettingConfigSection(ConfigSection):
+    """Custom subclass to automatically accommodate saving and retrieving values based on user profile settings.
+    """
+    PROFILES_KEY = 'user_profiles'
+    SETTINGS_KEY = 'user_profile_settings'
+
+    @classmethod
+    def init_profile_options(cls):
+        ListOption.add_if_missing('profiles', cls.PROFILES_KEY, [])
+        Option.add_if_missing('profiles', cls.SETTINGS_KEY, {})
+
+    def __init__(self, config, name):
+        super().__init__(config, name)
+        self.__qt_config = config
+        self.__name = name
+        self.__prefix = self.__name + '/'
+        self._memoization = defaultdict(Memovar)
+        self.init_profile_options()
+        self.profiles_override = None
+        self.settings_override = None
+
+    def _get_active_profile_ids(self):
+        if self.profiles_override is None:
+            profiles = self.__qt_config.profiles[self.PROFILES_KEY]
+        else:
+            profiles = self.profiles_override
+        if profiles is None:
+            return
+        for profile in profiles:
+            if profile['enabled']:
+                yield profile['id']
+
+    def _get_active_profile_settings(self):
+        for profile_id in self._get_active_profile_ids():
+            yield profile_id, self._get_profile_settings(profile_id)
+
+    def _get_profile_settings(self, profile_id):
+        if self.settings_override is None:
+            # Set to None if profile_id not in profile settings
+            profile_settings = self.__qt_config.profiles[self.SETTINGS_KEY][profile_id] if profile_id in self.__qt_config.profiles[self.SETTINGS_KEY] else None
+        else:
+            # Set to None if profile_id not in settings_override
+            profile_settings = self.settings_override[profile_id] if profile_id in self.settings_override else None
+        if profile_settings is None:
+            log.error("Unable to find settings for user profile '%s'", profile_id)
+            return {}
+        return profile_settings
+
+    def __getitem__(self, name):
+        # Don't process settings that are not profile-specific
+        if name in UserProfileGroups.ALL_SETTINGS:
+            for profile_id, settings in self._get_active_profile_settings():
+                if name in settings and settings[name] is not None:
+                    return settings[name]
+        opt = Option.get(self.__name, name)
+        if opt is None:
+            return None
+        return self.value(name, opt, opt.default)
+
+    def __setitem__(self, name, value):
+        # Don't process settings that are not profile-specific
+        if name in UserProfileGroups.ALL_SETTINGS:
+            for profile_id, settings in self._get_active_profile_settings():
+                if name in settings:
+                    self._save_profile_setting(profile_id, name, value)
+                    return
+        key = self.key(name)
+        self.__qt_config.setValue(key, value)
+        self._memoization[key].dirty = True
+
+    def _save_profile_setting(self, profile_id, name, value):
+        profile_settings = self.__qt_config.profiles[self.SETTINGS_KEY]
+        profile_settings[profile_id][name] = value
+        key = self.__qt_config.profiles.key(self.SETTINGS_KEY)
+        self.__qt_config.setValue(key, profile_settings)
+        self._memoization[key].dirty = True
+
+    def set_profiles_override(self, new_profiles=None):
+        self.profiles_override = new_profiles
+
+    def set_settings_override(self, new_settings=None):
+        self.settings_override = new_settings
+
+
 class Config(QtCore.QSettings):
 
     """Configuration.
@@ -132,23 +242,23 @@ class Config(QtCore.QSettings):
         :meth:`from_file`."""
 
         self.setAtomicSyncRequired(False)  # See comment in event()
-        self.application = ConfigSection(self, "application")
-        self.setting = ConfigSection(self, "setting")
-        self.persist = ConfigSection(self, "persist")
-        self.profile = ConfigSection(self, "profile/default")
-        self.current_preset = "default"
+        self.application = ConfigSection(self, 'application')
+        self.profiles = ConfigSection(self, 'profiles')
+        self.setting = SettingConfigSection(self, 'setting')
+        self.persist = ConfigSection(self, 'persist')
 
-        TextOption("application", "version", '0.0.0dev0')
-        self._version = Version.from_string(self.application["version"])
+        if 'version' not in self.application or not self.application['version']:
+            TextOption('application', 'version', '0.0.0dev0')
+        self._version = Version.from_string(self.application['version'])
         self._upgrade_hooks = dict()
 
     def event(self, event):
-        if event.type() == QtCore.QEvent.UpdateRequest:
+        if event.type() == QtCore.QEvent.Type.UpdateRequest:
             # Syncing the config file can trigger a deadlock between QSettings internal mutex and
             # the Python GIL in PyQt up to 5.15.2. Workaround this by handling this ourselves
             # with custom file locking.
             # See also https: // tickets.metabrainz.org/browse/PICARD-2088
-            log.debug('Config file update requested on thread %r', threading.get_ident())
+            log.debug("Config file update requested on thread %r", threading.get_ident())
             self.sync()
             return True
         else:
@@ -171,8 +281,8 @@ class Config(QtCore.QSettings):
         """Build a Config object using the default configuration file
         location."""
         this = cls()
-        QtCore.QSettings.__init__(this, QtCore.QSettings.IniFormat,
-                                  QtCore.QSettings.UserScope, PICARD_ORG_NAME,
+        QtCore.QSettings.__init__(this, QtCore.QSettings.Format.IniFormat,
+                                  QtCore.QSettings.Scope.UserScope, PICARD_ORG_NAME,
                                   PICARD_APP_NAME, parent)
 
         # Check if there is a config file specifically for this version
@@ -197,19 +307,11 @@ class Config(QtCore.QSettings):
         """Build a Config object using a user-provided configuration file
         path."""
         this = cls()
-        QtCore.QSettings.__init__(this, filename, QtCore.QSettings.IniFormat,
+        QtCore.QSettings.__init__(this, filename, QtCore.QSettings.Format.IniFormat,
                                   parent)
         this.__initialize()
         return this
 
-    def switchProfile(self, profilename):
-        """Sets the current profile."""
-        key = "profile/%s" % (profilename,)
-        if self.contains(key):
-            self.profile.name = key
-        else:
-            raise KeyError("Unknown profile '%s'" % (profilename,))
-
     def register_upgrade_hook(self, func, *args):
         """Register a function to upgrade from one config version to another"""
         to_version = Version.from_string(func.__name__)
@@ -265,7 +367,7 @@ class Config(QtCore.QSettings):
                 # hook is not applicable, mark as done
                 hook['done'] = True
 
-        if all(map(itemgetter("done"), self._upgrade_hooks.values())):
+        if all(map(itemgetter('done'), self._upgrade_hooks.values())):
             # all hooks were executed, ensure config is marked with latest version
             self._version = PICARD_VERSION
             self._write_version()
@@ -273,14 +375,19 @@ class Config(QtCore.QSettings):
     def _backup_settings(self):
         if Version(0, 0, 0) < self._version < PICARD_VERSION:
             backup_path = self._versioned_config_filename()
-            log.info('Backing up config file to %s', backup_path)
-            try:
-                shutil.copyfile(self.fileName(), backup_path)
-            except OSError:
-                log.error('Failed backing up config file to %s', backup_path)
+            self._save_backup(backup_path)
+
+    def _save_backup(self, backup_path):
+        log.info("Backing up config file to %s", backup_path)
+        try:
+            shutil.copyfile(self.fileName(), backup_path)
+        except OSError:
+            log.error("Failed backing up config file to %s", backup_path)
+            return False
+        return True
 
     def _write_version(self):
-        self.application["version"] = self._version.to_string()
+        self.application['version'] = self._version.to_string()
         self.sync()
 
     def _versioned_config_filename(self, version=None):
@@ -289,6 +396,12 @@ class Config(QtCore.QSettings):
         return os.path.join(os.path.dirname(self.fileName()), '%s-%s.ini' % (
             self.applicationName(), version.to_string(short=True)))
 
+    def save_user_backup(self, backup_path):
+        if backup_path == self.fileName():
+            log.warning("Attempt to backup configuration file to the same path.")
+            return False
+        return self._save_backup(backup_path)
+
 
 class Option(QtCore.QObject):
 
@@ -298,18 +411,41 @@ class Option(QtCore.QObject):
     qtype = None
 
     def __init__(self, section, name, default):
+        key = (section, name)
+        if key in self.registry:
+            stack = inspect.stack()
+            fmt = "Option %s/%s already declared"
+            args = [section, name]
+            if len(stack) > 1:
+                f = stack[1]
+                fmt += "\nat %s:%d: in %s"
+                args.extend((f.filename, f.lineno, f.function))
+                if f.code_context:
+                    fmt += "\n%s"
+                    args.append("\n".join(f.code_context).rstrip())
+            log.error(fmt, *args)
         super().__init__()
         self.section = section
         self.name = name
         self.default = default
-        if not hasattr(self, "convert"):
-            self.convert = type(default)
-        self.registry[(self.section, self.name)] = self
+        self.registry[key] = self
 
     @classmethod
     def get(cls, section, name):
         return cls.registry.get((section, name))
 
+    @classmethod
+    def add_if_missing(cls, section, name, default):
+        if not cls.exists(section, name):
+            cls(section, name, default)
+
+    @classmethod
+    def exists(cls, section, name):
+        return (section, name) in cls.registry
+
+    def convert(self, value):
+        return type(self.default)(value)
+
 
 class TextOption(Option):
 
@@ -342,21 +478,18 @@ class ListOption(Option):
 config = None
 setting = None
 persist = None
-
-_thread_configs = {}
-_thread_config_lock = threading.RLock()
+profiles = None
 
 
 def setup_config(app, filename=None):
-    global config, setting, persist
+    global config, setting, persist, profiles
     if filename is None:
         config = Config.from_app(app)
     else:
         config = Config.from_file(app, filename)
-    _thread_configs[threading.get_ident()] = config
     setting = config.setting
     persist = config.persist
-    _init_purge_config_timer()
+    profiles = config.profiles
 
 
 def get_config():
@@ -364,41 +497,15 @@ def get_config():
 
     Config objects for threads are created on demand and cached for later use.
     """
-    thread_id = threading.get_ident()
-    thread_config = _thread_configs.get(thread_id)
-    if not thread_config:
-        if not config:
-            return None  # Not yet initialized
-        _thread_config_lock.acquire()
-        try:
-            config_file = config.fileName()
-            log.debug('Instantiating Config for thread %s using %s.', thread_id, config_file)
-            thread_config = Config.from_file(None, config_file)
-            _thread_configs[thread_id] = thread_config
-        finally:
-            _thread_config_lock.release()
-    return thread_config
+    return config
 
 
-def _init_purge_config_timer(purge_interval_milliseconds=60000):
-    def run_purge_config_timer():
-        purge_config_instances()
-        start_purge_config_timer()
-
-    def start_purge_config_timer():
-        QtCore.QTimer.singleShot(purge_interval_milliseconds, run_purge_config_timer)
-
-    start_purge_config_timer()
-
-
-def purge_config_instances():
-    """Removes cached config instances for no longer active threads."""
-    _thread_config_lock.acquire()
+def load_new_config(filename=None):
+    config_file = get_config().fileName()
     try:
-        all_threads = set([thread.ident for thread in threading.enumerate()])
-        threads_config = set(_thread_configs)
-        for thread_id in threads_config.difference(all_threads):
-            log.debug('Purging config instance for thread %s.', thread_id)
-            del _thread_configs[thread_id]
-    finally:
-        _thread_config_lock.release()
+        shutil.copy(filename, config_file)
+    except OSError:
+        log.error("Failed restoring config file from %s", filename)
+        return False
+    setup_config(QtCore.QObject.tagger, config_file)
+    return True
