@@ -5,7 +5,7 @@
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2006-2008 Lukáš Lalinský
 # Copyright (C) 2011-2012 Michael Wiencek
-# Copyright (C) 2013, 2020-2021 Laurent Monin
+# Copyright (C) 2013, 2020-2021, 2023 Laurent Monin
 # Copyright (C) 2017 Sambhav Kothari
 # Copyright (C) 2017 Sophist-UK
 # Copyright (C) 2018 Vishal Choudhary
@@ -37,11 +37,16 @@ class DataObject(LockableObject):
     def __init__(self, obj_id):
         super().__init__()
         self.id = obj_id
-        self.genres = Counter()
         self.item = None
+        self._genres = Counter()
+
+    @property
+    def genres(self):
+        return self._genres
 
     def add_genre(self, name, count):
-        self.genres[name] += count
+        if name:
+            self._genres[name] += count
 
     @staticmethod
     def set_genre_inc_params(inc, config=None):
