@@ -3,11 +3,11 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2009, 2011 Lukáš Lalinský
-# Copyright (C) 2009-2011, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009-2011, 2018-2021, 2023 Philipp Wolfer
 # Copyright (C) 2011-2014 Wieland Hoffmann
 # Copyright (C) 2012-2013 Michael Wiencek
 # Copyright (C) 2013 Calvin Walton
-# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2017 Ville Skyttä
 #
@@ -49,6 +49,11 @@ from picard.util import (
     encode_filename,
     sanitize_date,
 )
+from picard.util.filenaming import (
+    get_available_filename,
+    move_ensure_casing,
+    replace_extension,
+)
 
 from .mutagenext import (
     aac,
@@ -57,8 +62,8 @@ from .mutagenext import (
 
 
 INVALID_CHARS = re.compile('[^\x20-\x7e]')
-DISALLOWED_KEYS = ['ID3', 'TAG', 'OggS', 'MP+']
-UNSUPPORTED_TAGS = [
+DISALLOWED_KEYS = {'ID3', 'TAG', 'OggS', 'MP+'}
+UNSUPPORTED_TAGS = {
     'gapless',
     'musicip_fingerprint',
     'podcast',
@@ -67,7 +72,7 @@ UNSUPPORTED_TAGS = [
     'showsort',
     'r128_album_gain',
     'r128_track_gain',
-]
+}
 
 
 def is_valid_key(key):
@@ -90,35 +95,35 @@ class APEv2File(File):
     _File = None
 
     __translate = {
-        "albumartist": "Album Artist",
-        "remixer": "MixArtist",
-        "director": "Director",
-        "website": "Weblink",
-        "discsubtitle": "DiscSubtitle",
-        "bpm": "BPM",
-        "isrc": "ISRC",
-        "catalognumber": "CatalogNumber",
-        "barcode": "Barcode",
-        "encodedby": "EncodedBy",
-        "language": "Language",
-        "movementnumber": "MOVEMENT",
-        "movement": "MOVEMENTNAME",
-        "movementtotal": "MOVEMENTTOTAL",
-        "showmovement": "SHOWMOVEMENT",
-        "releasestatus": "MUSICBRAINZ_ALBUMSTATUS",
-        "releasetype": "MUSICBRAINZ_ALBUMTYPE",
-        "musicbrainz_recordingid": "musicbrainz_trackid",
-        "musicbrainz_trackid": "musicbrainz_releasetrackid",
-        "originalartist": "Original Artist",
-        "replaygain_album_gain": "REPLAYGAIN_ALBUM_GAIN",
-        "replaygain_album_peak": "REPLAYGAIN_ALBUM_PEAK",
-        "replaygain_album_range": "REPLAYGAIN_ALBUM_RANGE",
-        "replaygain_track_gain": "REPLAYGAIN_TRACK_GAIN",
-        "replaygain_track_peak": "REPLAYGAIN_TRACK_PEAK",
-        "replaygain_track_range": "REPLAYGAIN_TRACK_RANGE",
-        "replaygain_reference_loudness": "REPLAYGAIN_REFERENCE_LOUDNESS",
+        'albumartist': 'Album Artist',
+        'remixer': 'MixArtist',
+        'director': 'Director',
+        'website': 'Weblink',
+        'discsubtitle': 'DiscSubtitle',
+        'bpm': 'BPM',
+        'isrc': 'ISRC',
+        'catalognumber': 'CatalogNumber',
+        'barcode': 'Barcode',
+        'encodedby': 'EncodedBy',
+        'language': 'Language',
+        'movementnumber': 'MOVEMENT',
+        'movement': 'MOVEMENTNAME',
+        'movementtotal': 'MOVEMENTTOTAL',
+        'showmovement': 'SHOWMOVEMENT',
+        'releasestatus': 'MUSICBRAINZ_ALBUMSTATUS',
+        'releasetype': 'MUSICBRAINZ_ALBUMTYPE',
+        'musicbrainz_recordingid': 'musicbrainz_trackid',
+        'musicbrainz_trackid': 'musicbrainz_releasetrackid',
+        'originalartist': 'Original Artist',
+        'replaygain_album_gain': 'REPLAYGAIN_ALBUM_GAIN',
+        'replaygain_album_peak': 'REPLAYGAIN_ALBUM_PEAK',
+        'replaygain_album_range': 'REPLAYGAIN_ALBUM_RANGE',
+        'replaygain_track_gain': 'REPLAYGAIN_TRACK_GAIN',
+        'replaygain_track_peak': 'REPLAYGAIN_TRACK_PEAK',
+        'replaygain_track_range': 'REPLAYGAIN_TRACK_RANGE',
+        'replaygain_reference_loudness': 'REPLAYGAIN_REFERENCE_LOUDNESS',
     }
-    __rtranslate = dict([(v.lower(), k) for k, v in __translate.items()])
+    __rtranslate = {v.lower(): k for k, v in __translate.items()}
 
     def __init__(self, filename):
         super().__init__(filename)
@@ -133,7 +138,7 @@ class APEv2File(File):
             for origname, values in file.tags.items():
                 name_lower = origname.lower()
                 if (values.kind == mutagen.apev2.BINARY
-                    and name_lower.startswith("cover art")):
+                    and name_lower.startswith('cover art')):
                     if b'\0' in values.value:
                         descr, data = values.value.split(b'\0', 1)
                         try:
@@ -143,8 +148,7 @@ class APEv2File(File):
                                 data=data,
                             )
                         except CoverArtImageError as e:
-                            log.error('Cannot load image from %r: %s' %
-                                      (filename, e))
+                            log.error("Cannot load image from %r: %s", filename, e)
                         else:
                             metadata.images.append(coverartimage)
 
@@ -153,22 +157,22 @@ class APEv2File(File):
                     continue
                 for value in values:
                     name = name_lower
-                    if name == "year":
-                        name = "date"
+                    if name == 'year':
+                        name = 'date'
                         value = sanitize_date(value)
-                    elif name == "track":
-                        name = "tracknumber"
-                        track = value.split("/")
+                    elif name == 'track':
+                        name = 'tracknumber'
+                        track = value.split('/')
                         if len(track) > 1:
-                            metadata["totaltracks"] = track[1]
+                            metadata['totaltracks'] = track[1]
                             value = track[0]
-                    elif name == "disc":
-                        name = "discnumber"
-                        disc = value.split("/")
+                    elif name == 'disc':
+                        name = 'discnumber'
+                        disc = value.split('/')
                         if len(disc) > 1:
-                            metadata["totaldiscs"] = disc[1]
+                            metadata['totaldiscs'] = disc[1]
                             value = disc[0]
-                    elif name in ('performer', 'comment'):
+                    elif name in {'performer', 'comment'}:
                         if value.endswith(')'):
                             start = value.rfind(' (')
                             if start > 0:
@@ -190,16 +194,19 @@ class APEv2File(File):
         except mutagen.apev2.APENoHeaderError:
             tags = mutagen.apev2.APEv2()
         images_to_save = list(metadata.images.to_be_saved_to_tags())
-        if config.setting["clear_existing_tags"]:
+        if config.setting['clear_existing_tags']:
+            preserved = []
+            if config.setting['preserve_images']:
+                preserved = list(self._iter_cover_art_tags(tags))
             tags.clear()
+            for name, value in preserved:
+                tags[name] = value
         elif images_to_save:
-            for name, value in tags.items():
-                if (value.kind == mutagen.apev2.BINARY
-                    and name.lower().startswith('cover art')):
-                    del tags[name]
+            for name, value in self._iter_cover_art_tags(tags):
+                del tags[name]
         temp = {}
         for name, value in metadata.items():
-            if name.startswith("~") or not self.supports_tag(name):
+            if name.startswith('~') or not self.supports_tag(name):
                 continue
             real_name = self._get_tag_name(name)
             # tracknumber/totaltracks => Track
@@ -210,7 +217,7 @@ class APEv2File(File):
             elif name == 'discnumber':
                 if 'totaldiscs' in metadata:
                     value = '%s/%s' % (value, metadata['totaldiscs'])
-            elif name in ('totaltracks', 'totaldiscs'):
+            elif name in {'totaltracks', 'totaldiscs'}:
                 continue
             # "performer:Piano=Joe Barr" => "Performer=Joe Barr (Piano)"
             elif name.startswith('performer:') or name.startswith('comment:'):
@@ -236,7 +243,7 @@ class APEv2File(File):
         """Remove the tags from the file that were deleted in the UI"""
         for tag in metadata.deleted_tags:
             real_name = self._get_tag_name(tag)
-            if real_name in ('Lyrics', 'Comment', 'Performer'):
+            if real_name in {'Lyrics', 'Comment', 'Performer'}:
                 parts = tag.split(':', 1)
                 if len(parts) == 2:
                     tag_type_regex = re.compile(r"\(%s\)$" % re.escape(parts[1]))
@@ -247,7 +254,7 @@ class APEv2File(File):
                     if re.search(tag_type_regex, item):
                         existing_tags.remove(item)
                 tags[real_name] = existing_tags
-            elif tag in ('totaltracks', 'totaldiscs'):
+            elif tag in {'totaltracks', 'totaldiscs'}:
                 tagstr = real_name.lower() + 'number'
                 if tagstr in metadata:
                     tags[real_name] = metadata[tagstr]
@@ -262,9 +269,9 @@ class APEv2File(File):
             return 'Lyrics'
         elif name == 'date':
             return 'Year'
-        elif name in ('tracknumber', 'totaltracks'):
+        elif name in {'tracknumber', 'totaltracks'}:
             return 'Track'
-        elif name in ('discnumber', 'totaldiscs'):
+        elif name in {'discnumber', 'totaldiscs'}:
             return 'Disc'
         elif name.startswith('performer:') or name.startswith('comment:'):
             return name.split(':', 1)[0].title()
@@ -273,9 +280,16 @@ class APEv2File(File):
         else:
             return name.title()
 
+    @staticmethod
+    def _iter_cover_art_tags(tags):
+        for name, value in tags.items():
+            if value.kind == mutagen.apev2.BINARY and name.lower().startswith('cover art'):
+                yield (name, value)
+
     @classmethod
     def supports_tag(cls, name):
         return (bool(name) and name not in UNSUPPORTED_TAGS
+                and not name.startswith('~')
                 and (is_valid_key(name)
                     or name.startswith('comment:')
                     or name.startswith('lyrics:')
@@ -301,14 +315,20 @@ class WavPackFile(APEv2File):
     NAME = "WavPack"
     _File = mutagen.wavpack.WavPack
 
-    def _save_and_rename(self, old_filename, metadata):
+    def _move_or_rename_wvc(self, old_filename, new_filename):
+        wvc_filename = replace_extension(old_filename, ".wvc")
+        if not isfile(wvc_filename):
+            return
+        wvc_new_filename = replace_extension(new_filename, ".wvc")
+        wvc_new_filename = get_available_filename(wvc_new_filename, wvc_filename)
+        log.debug('Moving Wavepack correction file %r => %r', wvc_filename, wvc_new_filename)
+        move_ensure_casing(wvc_filename, wvc_new_filename)
+
+    def _move_additional_files(self, old_filename, new_filename, config):
         """Includes an additional check for WavPack correction files"""
-        wvc_filename = old_filename.replace(".wv", ".wvc")
-        if isfile(wvc_filename):
-            config = get_config()
-            if config.setting["rename_files"] or config.setting["move_files"]:
-                self._rename(wvc_filename, metadata)
-        return File._save_and_rename(self, old_filename, metadata)
+        if config.setting['rename_files'] or config.setting['move_files']:
+            self._move_or_rename_wvc(old_filename, new_filename)
+        return super()._move_additional_files(old_filename, new_filename, config)
 
 
 class OptimFROGFile(APEv2File):
@@ -364,7 +384,7 @@ class AACFile(APEv2File):
             try:
                 mutagen.apev2.delete(encode_filename(filename))
             except BaseException:
-                log.exception('Error removing APEv2 tags from %s', filename)
+                log.exception("Error removing APEv2 tags from %s", filename)
 
     @classmethod
     def supports_tag(cls, name):
