@@ -42,7 +42,6 @@ from functools import reduce
 from inspect import getfullargspec
 import operator
 import re
-import unicodedata
 
 from picard.const.countries import RELEASE_COUNTRIES
 from picard.metadata import MULTI_VALUED_JOINER
@@ -54,6 +53,7 @@ from picard.script.parser import (
 )
 from picard.util import (
     pattern_as_regex,
+    titlecase,
     uniqify,
 )
 
@@ -64,7 +64,7 @@ except ImportError:
     markdown = None
 
 
-Bound = namedtuple("Bound", ["lower", "upper"])
+Bound = namedtuple('Bound', ['lower', 'upper'])
 
 
 class FunctionRegistryItem:
@@ -197,7 +197,7 @@ def func_if(parser, _if, _then, _else=None):
 
 
 @script_function(eval_args=False, documentation=N_(
-    """`$if2(a1,a2,a3,...)`
+    """`$if2(a1,a2,a3,…)`
 
 Returns first non empty argument."""
 ))
@@ -210,7 +210,7 @@ def func_if2(parser, *args):
 
 
 @script_function(eval_args=False, documentation=N_(
-    """`$noop(...)`
+    """`$noop(…)`
 
 Does nothing (useful for comments or disabling a block of code)."""
 ))
@@ -227,7 +227,7 @@ def func_left(parser, text, length):
     try:
         return text[:int(length)]
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -239,7 +239,7 @@ def func_right(parser, text, length):
     try:
         return text[-int(length):]
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -270,7 +270,7 @@ def func_pad(parser, text, length, char):
     try:
         return char * (int(length) - len(text)) + text
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -324,9 +324,9 @@ Returns true, if `x` contains `y`."""
 ))
 def func_in(parser, text, needle):
     if needle in text:
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(eval_args=False, documentation=N_(
@@ -368,13 +368,13 @@ def func_rsearch(parser, text, pattern):
     try:
         match = re.search(pattern, text)
     except re.error:
-        return ""
+        return ''
     if match:
         try:
             return match.group(1)
         except IndexError:
             return match.group(0)
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -386,7 +386,7 @@ def func_num(parser, text, length):
     try:
         format_ = "%%0%dd" % max(0, min(int(length), 20))
     except ValueError:
-        return ""
+        return ''
     try:
         value = int(text)
     except ValueError:
@@ -409,9 +409,9 @@ def func_unset(parser, name):
         for key in list(parser.context.keys()):
             if key.startswith(name):
                 parser.context.unset(key)
-        return ""
+        return ''
     parser.context.unset(name)
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -426,7 +426,7 @@ _Since Picard 2.1_"""
 ))
 def func_delete(parser, name):
     parser.context.delete(normalize_tagname(name))
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -444,7 +444,7 @@ def func_set(parser, name, value):
         parser.context[normalize_tagname(name)] = value
     else:
         func_unset(parser, name)
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -488,7 +488,7 @@ def func_copy(parser, new, old):
     new = normalize_tagname(new)
     old = normalize_tagname(old)
     parser.context[new] = parser.context.getall(old)[:]
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -508,7 +508,7 @@ def func_copymerge(parser, new, old, keep_duplicates=False):
     newvals = parser.context.getall(new)
     oldvals = parser.context.getall(old)
     parser.context[new] = newvals + oldvals if keep_duplicates else uniqify(newvals + oldvals)
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -525,7 +525,7 @@ def func_trim(parser, text, char=None):
 
 
 @script_function(documentation=N_(
-    """`$add(x,y,...)`
+    """`$add(x,y,…)`
 
 Add `y` to `x`.
 Can be used with an arbitrary number of arguments.
@@ -539,11 +539,11 @@ def func_add(parser, x, y, *args):
     try:
         return _compute_int(operator.add, x, y, *args)
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$sub(x,y,...)`
+    """`$sub(x,y,…)`
 
 Subtracts `y` from `x`.
 Can be used with an arbitrary number of arguments.
@@ -557,11 +557,11 @@ def func_sub(parser, x, y, *args):
     try:
         return _compute_int(operator.sub, x, y, *args)
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$div(x,y,...)`
+    """`$div(x,y,…)`
 
 Divides `x` by `y`.
 Can be used with an arbitrary number of arguments.
@@ -575,13 +575,13 @@ def func_div(parser, x, y, *args):
     try:
         return _compute_int(operator.floordiv, x, y, *args)
     except ValueError:
-        return ""
+        return ''
     except ZeroDivisionError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$mod(x,y,...)`
+    """`$mod(x,y,…)`
 
 Returns the remainder of `x` divided by `y`.
 Can be used with an arbitrary number of arguments.
@@ -595,11 +595,11 @@ def func_mod(parser, x, y, *args):
     try:
         return _compute_int(operator.mod, x, y, *args)
     except (ValueError, ZeroDivisionError):
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$mul(x,y,...)`
+    """`$mul(x,y,…)`
 
 Multiplies `x` by `y`.
 Can be used with an arbitrary number of arguments.
@@ -613,11 +613,11 @@ def func_mul(parser, x, y, *args):
     try:
         return _compute_int(operator.mul, x, y, *args)
     except ValueError:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$or(x,y,...)`
+    """`$or(x,y,…)`
 
 Returns true if either `x` or `y` not empty.
     Can be used with an arbitrary number of arguments.
@@ -625,13 +625,13 @@ Returns true if either `x` or `y` not empty.
 ))
 def func_or(parser, x, y, *args):
     if _compute_logic(any, x, y, *args):
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
-    """`$and(x,y,...)`
+    """`$and(x,y,…)`
 
 Returns true if both `x` and `y` are not empty.
     Can be used with an arbitrary number of arguments.
@@ -639,9 +639,9 @@ Returns true if both `x` and `y` are not empty.
 ))
 def func_and(parser, x, y, *args):
     if _compute_logic(all, x, y, *args):
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -651,9 +651,9 @@ Returns true if `x` is empty."""
 ))
 def func_not(parser, x):
     if not x:
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -663,9 +663,9 @@ Returns true if `x` equals `y`."""
 ))
 def func_eq(parser, x, y):
     if x == y:
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -675,9 +675,9 @@ Returns true if `x` does not equal `y`."""
 ))
 def func_ne(parser, x, y):
     if x != y:
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 def _cmp(op, x, y, _type):
@@ -697,9 +697,9 @@ def _cmp(op, x, y, _type):
             except ValueError:
                 pass
     if _type == 'text':
-        return "1" if op(x, y) else ""
+        return '1' if op(x, y) else ""
     elif _type == 'nocase':
-        return "1" if op(x.lower(), y.lower()) else ""
+        return '1' if op(x.lower(), y.lower()) else ""
     elif _type == 'float':
         _typer = float
     elif _type == 'int':
@@ -707,10 +707,10 @@ def _cmp(op, x, y, _type):
     if _typer is not None:
         try:
             if op(_typer(x), _typer(y)):
-                return "1"
+                return '1'
         except ValueError:
             pass
-    return ""
+    return ''
 
 
 @script_function(documentation=N_(
@@ -836,7 +836,7 @@ def func_matchedtracks(parser, *args):
     file = parser.file
     if file and file.parent and hasattr(file.parent, 'album') and file.parent.album:
         return str(parser.file.parent.album.get_num_matched_tracks())
-    return "0"
+    return '0'
 
 
 @script_function(documentation=N_(
@@ -850,8 +850,8 @@ def func_is_complete(parser):
     file = parser.file
     if (file and file.parent and hasattr(file.parent, 'album') and file.parent.album
             and file.parent.album.is_complete()):
-        return "1"
-    return ""
+        return '1'
+    return ''
 
 
 @script_function(documentation=N_(
@@ -881,7 +881,7 @@ Returns the first character of each word in `text`, if it is an alphabetic chara
 _Since Picard 0.12_"""
 ))
 def func_initials(parser, text=""):
-    return "".join(a[:1] for a in text.split(" ") if a[:1].isalpha())
+    return ''.join(a[:1] for a in text.split(" ") if a[:1].isalpha())
 
 
 @script_function(documentation=N_(
@@ -916,8 +916,8 @@ _Since Picard 1.4_"""
 ))
 def func_startswith(parser, text, prefix):
     if text.startswith(prefix):
-        return "1"
-    return ""
+        return '1'
+    return ''
 
 
 @script_function(documentation=N_(
@@ -929,8 +929,8 @@ _Since Picard 1.4_"""
 ))
 def func_endswith(parser, text, suffix):
     if text.endswith(suffix):
-        return "1"
-    return ""
+        return '1'
+    return ''
 
 
 @script_function(documentation=N_(
@@ -949,7 +949,7 @@ def func_truncate(parser, text, length):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$swapprefix(text,prefix1,prefix2,...)`
+    """`$swapprefix(text,prefix1,prefix2,…)`
 
 Moves the specified prefixes from the beginning to the end of `text`. Multiple
 prefixes can be specified as separate parameters. If no prefix is specified 'A'
@@ -971,7 +971,7 @@ def func_swapprefix(parser, text, *prefixes):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$delprefix(text,prefix1,prefix2,...)`
+    """`$delprefix(text,prefix1,prefix2,…)`
 
 Deletes the specified prefixes from the beginning of `text`. Multiple
 prefixes can be specified as separate parameters.  If no prefix is specified 'A'
@@ -1009,10 +1009,10 @@ def _delete_prefix(parser, text, *prefixes):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$eq_any(x,a1,a2,...)`
+    """`$eq_any(x,a1,a2,…)`
 
-Returns true if `x` equals `a1` or `a2` or ...
-Functionally equivalent to `$or($eq(x,a1),$eq(x,a2),...)`.
+Returns true if `x` equals `a1` or `a2` or …
+Functionally equivalent to `$or($eq(x,a1),$eq(x,a2),…)`.
 Functionally equivalent to the eq2 plugin."""
 ))
 def func_eq_any(parser, x, *args):
@@ -1021,10 +1021,10 @@ def func_eq_any(parser, x, *args):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$ne_all(x,a1,a2,...)`
+    """`$ne_all(x,a1,a2,…)`
 
-Returns true if `x` does not equal `a1` and `a2` and ...
-Functionally equivalent to `$and($ne(x,a1),$ne(x,a2),...)`.
+Returns true if `x` does not equal `a1` and `a2` and …
+Functionally equivalent to `$and($ne(x,a1),$ne(x,a2),…)`.
 Functionally equivalent to the ne2 plugin."""
 ))
 def func_ne_all(parser, x, *args):
@@ -1033,10 +1033,10 @@ def func_ne_all(parser, x, *args):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$eq_all(x,a1,a2,...)`
+    """`$eq_all(x,a1,a2,…)`
 
-Returns true if `x` equals `a1` and `a2` and ...
-Functionally equivalent to `$and($eq(x,a1),$eq(x,a2),...)`.
+Returns true if `x` equals `a1` and `a2` and …
+Functionally equivalent to `$and($eq(x,a1),$eq(x,a2),…)`.
 
 Example:
 
@@ -1051,10 +1051,10 @@ def func_eq_all(parser, x, *args):
 
 
 @script_function(check_argcount=False, documentation=N_(
-    """`$ne_any(x,a1,a2,...)`
+    """`$ne_any(x,a1,a2,…)`
 
-Returns true if `x` does not equal `a1` or `a2` or ...
-Functionally equivalent to `$or($ne(x,a1),$ne(x,a2),...)`.
+Returns true if `x` does not equal `a1` or `a2` or …
+Functionally equivalent to `$or($ne(x,a1),$ne(x,a2),…)`.
 
 Example:
 
@@ -1076,33 +1076,7 @@ Example:
 _Since Picard 2.1_"""
 ))
 def func_title(parser, text):
-    # GPL 2.0 licensed code by Javier Kohen, Sambhav Kothari
-    # from https://github.com/metabrainz/picard-plugins/blob/2.0/plugins/titlecase/titlecase.py
-    if not text:
-        return text
-    capitalized = text[0].capitalize()
-    capital = False
-    for i in range(1, len(text)):
-        t = text[i]
-        if t in "’'" and text[i-1].isalpha():
-            capital = False
-        elif iswbound(t):
-            capital = True
-        elif capital and t.isalpha():
-            capital = False
-            t = t.capitalize()
-        else:
-            capital = False
-        capitalized += t
-    return capitalized
-
-
-def iswbound(char):
-    # GPL 2.0 licensed code by Javier Kohen, Sambhav Kothari
-    # from https://github.com/metabrainz/picard-plugins/blob/2.0/plugins/titlecase/titlecase.py
-    """ Checks whether the given character is a word boundary """
-    category = unicodedata.category(char)
-    return "Zs" == category or "Sk" == category or "P" == category[0]
+    return titlecase(text)
 
 
 @script_function(documentation=N_(
@@ -1114,9 +1088,9 @@ _Since Picard 2.2_"""
 ))
 def func_is_audio(parser):
     if func_is_video(parser) == "1":
-        return ""
+        return ''
     else:
-        return "1"
+        return '1'
 
 
 @script_function(documentation=N_(
@@ -1128,9 +1102,9 @@ _Since Picard 2.2_"""
 ))
 def func_is_video(parser):
     if parser.context['~video'] and parser.context['~video'] != '0':
-        return "1"
+        return '1'
     else:
-        return ""
+        return ''
 
 
 @script_function(documentation=N_(
@@ -1553,7 +1527,7 @@ def func_cleanmulti(parser, multi):
     name = normalize_tagname(multi)
     values = [str(value) for value in parser.context.getall(name) if value or value == 0]
     parser.context[name] = values
-    return ""
+    return ''
 
 
 def _type_args(_type, *args):
@@ -1592,7 +1566,7 @@ def _extract(_func, _type, *args):
     try:
         haystack = _type_args(_type, *args)
     except ValueError:
-        return ""
+        return ''
 
     if _type == 'nocase':
         op = operator.lt if _func == min else operator.gt
@@ -1606,7 +1580,7 @@ def _extract(_func, _type, *args):
 
 
 @script_function(documentation=N_(
-    """`$min(type,x,...)`
+    """`$min(type,x,…)`
 
 Returns the minimum value using the comparison specified in `type`.
 
@@ -1627,7 +1601,7 @@ def func_min(parser, _type, x, *args):
 
 
 @script_function(documentation=N_(
-    """`$max(type,x,...)`
+    """`$max(type,x,…)`
 
 Returns the maximum value using the comparison specified in `type`.
 
