@@ -131,6 +131,7 @@ from picard.ui.passworddialog import (
     PasswordDialog,
     ProxyDialog,
 )
+from picard.ui.pluginupdatedialog import PluginUpdatesDialog
 from picard.ui.savewarningdialog import SaveWarningDialog
 from picard.ui.scripteditor import (
     ScriptEditorDialog,
@@ -176,15 +177,15 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     ready_for_display = QtCore.pyqtSignal()
 
     options = [
-        Option("persist", "window_state", QtCore.QByteArray()),
-        BoolOption("persist", "window_maximized", False),
-        BoolOption("persist", "view_metadata_view", True),
-        BoolOption("persist", "view_cover_art", True),
-        BoolOption("persist", "view_toolbar", True),
-        BoolOption("persist", "view_file_browser", False),
-        TextOption("persist", "current_directory", ""),
-        FloatOption("persist", "mediaplayer_playback_rate", 1.0),
-        IntOption("persist", "mediaplayer_volume", 50),
+        Option('persist', 'window_state', QtCore.QByteArray()),
+        BoolOption('persist', 'window_maximized', False),
+        BoolOption('persist', 'view_metadata_view', True),
+        BoolOption('persist', 'view_cover_art', True),
+        BoolOption('persist', 'view_toolbar', True),
+        BoolOption('persist', 'view_file_browser', False),
+        TextOption('persist', 'current_directory', ""),
+        FloatOption('persist', 'mediaplayer_playback_rate', 1.0),
+        IntOption('persist', 'mediaplayer_volume', 50),
     ]
 
     def __init__(self, parent=None, disable_player=False):
@@ -208,7 +209,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 self.player.error.connect(self._on_player_error)
 
         self.script_editor_dialog = None
-        self.examples = None
+
+        self.tagger.pluginmanager.updates_available.connect(self.show_plugin_update_dialog)
 
         self.check_and_repair_naming_scripts()
         self.check_and_repair_profiles()
@@ -313,7 +315,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def closeEvent(self, event):
         config = get_config()
-        if config.setting["quit_confirmation"] and not self.show_quit_confirmation():
+        if config.setting['quit_confirmation'] and not self.show_quit_confirmation():
             event.ignore()
             return
         if self.player:
@@ -322,7 +324,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.saveWindowState()
         # Confirm loss of unsaved changes in script editor.
         if self.script_editor_dialog:
-            if not self.script_editor_dialog.unsaved_changes_confirmation():
+            if not self.script_editor_dialog.unsaved_changes_in_profile_confirmation():
                 event.ignore()
                 return
             else:
@@ -364,14 +366,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def saveWindowState(self):
         config = get_config()
-        config.persist["window_state"] = self.saveState()
+        config.persist['window_state'] = self.saveState()
         isMaximized = int(self.windowState()) & QtCore.Qt.WindowState.WindowMaximized != 0
         self.save_geometry()
-        config.persist["window_maximized"] = isMaximized
-        config.persist["view_metadata_view"] = self.show_metadata_view_action.isChecked()
-        config.persist["view_cover_art"] = self.show_cover_art_action.isChecked()
-        config.persist["view_toolbar"] = self.show_toolbar_action.isChecked()
-        config.persist["view_file_browser"] = self.show_file_browser_action.isChecked()
+        config.persist['window_maximized'] = isMaximized
+        config.persist['view_metadata_view'] = self.show_metadata_view_action.isChecked()
+        config.persist['view_cover_art'] = self.show_cover_art_action.isChecked()
+        config.persist['view_toolbar'] = self.show_toolbar_action.isChecked()
+        config.persist['view_file_browser'] = self.show_file_browser_action.isChecked()
         self.file_browser.save_state()
         self.panel.save_state()
         self.metadata_box.save_state()
@@ -379,11 +381,11 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     @restore_method
     def restoreWindowState(self):
         config = get_config()
-        self.restoreState(config.persist["window_state"])
+        self.restoreState(config.persist['window_state'])
         self.restore_geometry()
-        if config.persist["window_maximized"]:
+        if config.persist['window_maximized']:
             self.setWindowState(QtCore.Qt.WindowState.WindowMaximized)
-        splitters = config.persist["splitters_MainWindow"]
+        splitters = config.persist['splitters_MainWindow']
         if splitters is None or 'main_window_bottom_splitter' not in splitters:
             self.centralWidget().setSizes([366, 194])
         self.file_browser.restore_state()
@@ -394,6 +396,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         infostatus = InfoStatus(self)
         self._progress = infostatus.get_progress
         self.listening_label = QtWidgets.QLabel()
+        self.listening_label.setStyleSheet("QLabel { margin: 0 4px 0 4px; }")
         self.listening_label.setVisible(False)
         self.listening_label.setToolTip("<qt/>" + _(
             "Picard listens on this port to integrate with your browser. When "
@@ -421,7 +424,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def update_statusbar_listen_port(self, listen_port):
         if listen_port:
             self.listening_label.setVisible(True)
-            self.listening_label.setText(_(" Listening on port %(port)d ") % {"port": listen_port})
+            self.listening_label.setText(_("Listening on port %(port)d") % {"port": listen_port})
         else:
             self.listening_label.setVisible(False)
 
@@ -476,7 +479,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _on_submit_acoustid(self):
         if self.tagger.use_acoustid:
             config = get_config()
-            if not config.setting["acoustid_apikey"]:
+            if not config.setting['acoustid_apikey']:
                 msg = QtWidgets.QMessageBox(self)
                 msg.setIcon(QtWidgets.QMessageBox.Icon.Information)
                 msg.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
@@ -489,20 +492,20 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 msg.addButton(open_options, QtWidgets.QMessageBox.ButtonRole.YesRole)
                 msg.exec_()
                 if msg.clickedButton() == open_options:
-                    self.show_options("fingerprinting")
+                    self.show_options('fingerprinting')
             else:
                 self.tagger.acoustidmanager.submit()
 
     @MainWindowActions.add()
     def _create_options_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('preferences-desktop'), _("&Options..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('preferences-desktop'), _("&Options…"), self)
         action.setMenuRole(QtWidgets.QAction.MenuRole.PreferencesRole)
         action.triggered.connect(self.show_options)
         self.options_action = action
 
     @MainWindowActions.add()
     def _create_show_script_editor_action(self):
-        action = QtWidgets.QAction(_("Open &file naming script editor..."))
+        action = QtWidgets.QAction(_("Open &file naming script editor…"))
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+S")))
         action.triggered.connect(self.open_file_naming_script_editor)
         self.show_script_editor_action = action
@@ -525,50 +528,50 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_help_action(self):
-        action = QtWidgets.QAction(_("&Help..."), self)
+        action = QtWidgets.QAction(_("&Help…"), self)
         action.setShortcut(QtGui.QKeySequence.StandardKey.HelpContents)
         action.triggered.connect(self.show_help)
         self.help_action = action
 
     @MainWindowActions.add()
     def _create_about_action(self):
-        action = QtWidgets.QAction(_("&About..."), self)
+        action = QtWidgets.QAction(_("&About…"), self)
         action.setMenuRole(QtWidgets.QAction.MenuRole.AboutRole)
         action.triggered.connect(self.show_about)
         self.about_action = action
 
     @MainWindowActions.add()
     def _create_donate_action(self):
-        action = QtWidgets.QAction(_("&Donate..."), self)
+        action = QtWidgets.QAction(_("&Donate…"), self)
         action.triggered.connect(self.open_donation_page)
         self.donate_action = action
 
     @MainWindowActions.add()
     def _create_report_bug_action(self):
-        action = QtWidgets.QAction(_("&Report a Bug..."), self)
+        action = QtWidgets.QAction(_("&Report a Bug…"), self)
         action.triggered.connect(self.open_bug_report)
         self.report_bug_action = action
 
     @MainWindowActions.add()
     def _create_support_forum_action(self):
-        action = QtWidgets.QAction(_("&Support Forum..."), self)
+        action = QtWidgets.QAction(_("&Support Forum…"), self)
         action.triggered.connect(self.open_support_forum)
         self.support_forum_action = action
 
     @MainWindowActions.add()
     def _create_add_files_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('document-open'), _("&Add Files..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('document-open'), _("&Add Files…"), self)
         action.setStatusTip(_("Add files to the tagger"))
-        # TR: Keyboard shortcut for "Add Files..."
+        # TR: Keyboard shortcut for "Add Files…"
         action.setShortcut(QtGui.QKeySequence.StandardKey.Open)
         action.triggered.connect(self.add_files)
         self.add_files_action = action
 
     @MainWindowActions.add()
     def _create_add_directory_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('folder'), _("Add Fold&er..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('folder'), _("Add Fold&er…"), self)
         action.setStatusTip(_("Add a folder to the tagger"))
-        # TR: Keyboard shortcut for "Add Directory..."
+        # TR: Keyboard shortcut for "Add Directory…"
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+E")))
         action.triggered.connect(self.add_directory)
         self.add_directory_action = action
@@ -631,7 +634,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     @MainWindowActions.add()
     def _create_submit_cluster_action(self):
         if addrelease.is_available():
-            action = QtWidgets.QAction(_("Submit cluster as release..."), self)
+            action = QtWidgets.QAction(_("Submit cluster as release…"), self)
             action.setStatusTip(_("Submit cluster as a new release to MusicBrainz"))
             action.setEnabled(False)
             action.triggered.connect(self.submit_cluster)
@@ -642,7 +645,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     @MainWindowActions.add()
     def _create_submit_file_as_recording_action(self):
         if addrelease.is_available():
-            action = QtWidgets.QAction(_("Submit file as standalone recording..."), self)
+            action = QtWidgets.QAction(_("Submit file as standalone recording…"), self)
             action.setStatusTip(_("Submit file as a new recording to MusicBrainz"))
             action.setEnabled(False)
             action.triggered.connect(self.submit_file)
@@ -653,7 +656,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     @MainWindowActions.add()
     def _create_submit_file_as_release_action(self):
         if addrelease.is_available():
-            action = QtWidgets.QAction(_("Submit file as release..."), self)
+            action = QtWidgets.QAction(_("Submit file as release…"), self)
             action.setStatusTip(_("Submit file as a new release to MusicBrainz"))
             action.setEnabled(False)
             action.triggered.connect(partial(self.submit_file, as_release=True))
@@ -663,7 +666,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_similar_items_search_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar items..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar items…"), self)
         action.setIconText(_("Similar items"))
         action.setStatusTip(_("View similar releases or recordings and optionally choose a different one"))
         action.setEnabled(False)
@@ -673,7 +676,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_album_search_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar albums..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar albums…"), self)
         action.setStatusTip(_("View similar releases and optionally choose a different release"))
         action.setEnabled(False)
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+T")))
@@ -682,7 +685,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_track_search_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar tracks..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar tracks…"), self)
         action.setStatusTip(_("View similar tracks and optionally choose a different release"))
         action.setEnabled(False)
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+T")))
@@ -691,7 +694,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_album_other_versions_action(self):
-        action = QtWidgets.QAction(_("Show &other album versions..."), self)
+        action = QtWidgets.QAction(_("Show &other album versions…"), self)
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+O")))
         action.triggered.connect(self.show_album_other_versions)
         self.album_other_versions_action = action
@@ -701,7 +704,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("File &Browser"), self)
         action.setCheckable(True)
-        if config.persist["view_file_browser"]:
+        if config.persist['view_file_browser']:
             action.setChecked(True)
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+B")))
         action.triggered.connect(self.show_file_browser)
@@ -712,7 +715,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("&Metadata"), self)
         action.setCheckable(True)
-        if config.persist["view_metadata_view"]:
+        if config.persist['view_metadata_view']:
             action.setChecked(True)
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+M")))
         action.triggered.connect(self.show_metadata_view)
@@ -723,7 +726,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("&Cover Art"), self)
         action.setCheckable(True)
-        if config.persist["view_cover_art"]:
+        if config.persist['view_cover_art']:
             action.setChecked(True)
         action.setEnabled(self.show_metadata_view_action.isChecked())
         action.triggered.connect(self.show_cover_art)
@@ -734,7 +737,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("&Actions"), self)
         action.setCheckable(True)
-        if config.persist["view_toolbar"]:
+        if config.persist['view_toolbar']:
             action.setChecked(True)
         action.triggered.connect(self.show_toolbar)
         self.show_toolbar_action = action
@@ -748,14 +751,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_cd_lookup_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('media-optical'), _("Lookup &CD..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('media-optical'), _("Lookup &CD…"), self)
         action.setStatusTip(_("Lookup the details of the CD in your drive"))
         # TR: Keyboard shortcut for "Lookup CD"
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+K")))
         action.triggered.connect(self.tagger.lookup_cd)
         self.cd_lookup_action = action
 
-        menu = QtWidgets.QMenu(_("Lookup &CD..."))
+        menu = QtWidgets.QMenu(_("Lookup &CD…"))
         menu.setIcon(icontheme.lookup('media-optical'))
         menu.triggered.connect(self.tagger.lookup_cd)
         self.cd_lookup_menu = menu
@@ -771,7 +774,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         action = QtWidgets.QAction(icontheme.lookup('picard-analyze'), _("&Scan"), self)
         action.setStatusTip(_("Use AcoustID audio fingerprint to identify the files by the actual music, even if they have no metadata"))
         action.setEnabled(False)
-        action.setToolTip(_('Identify the file using its AcoustID audio fingerprint'))
+        action.setToolTip(_("Identify the file using its AcoustID audio fingerprint"))
         # TR: Keyboard shortcut for "Analyze"
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Y")))
         action.triggered.connect(self.analyze)
@@ -783,7 +786,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         action.setIconText(_("Generate Fingerprints"))
         action.setStatusTip(_("Generate the AcoustID audio fingerprints for the selected files without doing a lookup"))
         action.setEnabled(False)
-        action.setToolTip(_('Generate the AcoustID audio fingerprints for the selected files'))
+        action.setToolTip(_("Generate the AcoustID audio fingerprints for the selected files"))
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+Y")))
         action.triggered.connect(self.generate_fingerprints)
         self.generate_fingerprints_action = action
@@ -812,7 +815,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     @MainWindowActions.add()
     def _create_view_info_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('picard-edit-tags'), _("&Info..."), self)
+        action = QtWidgets.QAction(icontheme.lookup('picard-edit-tags'), _("&Info…"), self)
         action.setEnabled(False)
         # TR: Keyboard shortcut for "Info"
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+I")))
@@ -831,7 +834,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("&Rename Files"), self)
         action.setCheckable(True)
-        action.setChecked(config.setting["rename_files"])
+        action.setChecked(config.setting['rename_files'])
         action.triggered.connect(self.toggle_rename_files)
         self.enable_renaming_action = action
 
@@ -840,7 +843,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("&Move Files"), self)
         action.setCheckable(True)
-        action.setChecked(config.setting["move_files"])
+        action.setChecked(config.setting['move_files'])
         action.triggered.connect(self.toggle_move_files)
         self.enable_moving_action = action
 
@@ -849,15 +852,15 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         action = QtWidgets.QAction(_("Save &Tags"), self)
         action.setCheckable(True)
-        action.setChecked(not config.setting["dont_write_tags"])
+        action.setChecked(not config.setting['dont_write_tags'])
         action.triggered.connect(self.toggle_tag_saving)
         self.enable_tag_saving_action = action
 
     @MainWindowActions.add()
     def _create_tags_from_filenames_action(self):
-        action = QtWidgets.QAction(icontheme.lookup('picard-tags-from-filename'), _("Tags From &File Names..."), self)
-        action.setIconText(_("Parse File Names..."))
-        action.setToolTip(_('Set tags based on the file names'))
+        action = QtWidgets.QAction(icontheme.lookup('picard-tags-from-filename'), _("Tags From &File Names…"), self)
+        action.setIconText(_("Parse File Names…"))
+        action.setToolTip(_("Set tags based on the file names"))
         action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+T")))
         action.setEnabled(False)
         action.triggered.connect(self.open_tags_from_filenames)
@@ -867,7 +870,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _create_open_collection_in_browser_action(self):
         config = get_config()
         action = QtWidgets.QAction(_("&Open My Collections in Browser"), self)
-        action.setEnabled(config.setting["username"] != '')
+        action.setEnabled(config.setting['username'] != '')
         action.triggered.connect(self.open_collection_in_browser)
         self.open_collection_in_browser_action = action
 
@@ -934,7 +937,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             log.warning(DISCID_NOT_LOADED_MESSAGE)
         else:
             config = get_config()
-            shortcut_drive = config.setting["cd_lookup_device"].split(",")[0] if len(drives) > 1 else ""
+            shortcut_drive = config.setting['cd_lookup_device'].split(",")[0] if len(drives) > 1 else ""
             for drive in drives:
                 action = self.cd_lookup_menu.addAction(drive)
                 action.setData(drive)
@@ -946,7 +949,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _set_cd_lookup_from_file_actions(self, drives):
         if self.cd_lookup_menu.actions():
             self.cd_lookup_menu.addSeparator()
-        action = self.cd_lookup_menu.addAction(_('From CD ripper &log file...'))
+        action = self.cd_lookup_menu.addAction(_("From CD ripper &log file…"))
         if not drives:
             self._update_cd_lookup_default_action(action)
         action.setData('logfile:eac')
@@ -970,17 +973,17 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def toggle_rename_files(self, checked):
         config = get_config()
-        config.setting["rename_files"] = checked
+        config.setting['rename_files'] = checked
         self.update_script_editor_examples()
 
     def toggle_move_files(self, checked):
         config = get_config()
-        config.setting["move_files"] = checked
+        config.setting['move_files'] = checked
         self.update_script_editor_examples()
 
     def toggle_tag_saving(self, checked):
         config = get_config()
-        config.setting["dont_write_tags"] = not checked
+        config.setting['dont_write_tags'] = not checked
 
     def get_selected_or_unmatched_files(self):
         if self.selected_objects:
@@ -1080,7 +1083,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def update_toolbar_style(self):
         config = get_config()
         style = QtCore.Qt.ToolButtonStyle.ToolButtonIconOnly
-        if config.setting["toolbar_show_labels"]:
+        if config.setting['toolbar_show_labels']:
             style = QtCore.Qt.ToolButtonStyle.ToolButtonTextUnderIcon
         self.toolbar.setToolButtonStyle(style)
         if self.player:
@@ -1099,7 +1102,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.removeToolBar(self.toolbar)
         self.toolbar = toolbar = QtWidgets.QToolBar(_("Actions"))
         self.insertToolBar(self.search_toolbar, self.toolbar)
-        toolbar.setObjectName("main_toolbar")
+        toolbar.setObjectName('main_toolbar')
         if self._is_wayland:
             toolbar.setFloatable(False)  # https://bugreports.qt.io/browse/QTBUG-92191
         if IS_MACOS:
@@ -1122,7 +1125,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 try:
                     add_toolbar_action(getattr(self, action))
                 except AttributeError:
-                    log.warning('Warning: Unknown action name "%r" found in config. Ignored.', action)
+                    log.warning("Warning: Unknown action name '%r' found in config. Ignored.", action)
         self.show_toolbar()
 
     def create_player_toolbar(self):
@@ -1138,7 +1141,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         self.search_toolbar = toolbar = self.addToolBar(_("Search"))
         self.search_toolbar_toggle_action = self.search_toolbar.toggleViewAction()
-        toolbar.setObjectName("search_toolbar")
+        toolbar.setObjectName('search_toolbar')
         if self._is_wayland:
             toolbar.setFloatable(False)  # https://bugreports.qt.io/browse/QTBUG-92191
         if IS_MACOS:
@@ -1147,9 +1150,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         search_panel = QtWidgets.QWidget(toolbar)
         hbox = QtWidgets.QHBoxLayout(search_panel)
         self.search_combo = QtWidgets.QComboBox(search_panel)
-        self.search_combo.addItem(_("Album"), "album")
-        self.search_combo.addItem(_("Artist"), "artist")
-        self.search_combo.addItem(_("Track"), "track")
+        self.search_combo.addItem(_("Album"), 'album')
+        self.search_combo.addItem(_("Artist"), 'artist')
+        self.search_combo.addItem(_("Track"), 'track')
         hbox.addWidget(self.search_combo, 0)
         self.search_edit = QtWidgets.QLineEdit(search_panel)
         self.search_edit.setClearButtonEnabled(True)
@@ -1239,7 +1242,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         entity = self.search_combo.itemData(self.search_combo.currentIndex())
         config = get_config()
         self.tagger.search(text, entity,
-                           config.setting["use_adv_search_syntax"],
+                           config.setting['use_adv_search_syntax'],
                            mbid_matched_callback=self.search_mbid_found)
 
     def add_files(self):
@@ -1267,7 +1270,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         files, _filter = QtWidgets.QFileDialog.getOpenFileNames(self, "", current_directory, ";;".join(formats))
         if files:
             config = get_config()
-            config.persist["current_directory"] = os.path.dirname(files[0])
+            config.persist['current_directory'] = os.path.dirname(files[0])
             self.tagger.add_files(files)
 
     def add_directory(self):
@@ -1276,7 +1279,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
         dir_list = []
         config = get_config()
-        if not config.setting["toolbar_multiselect"]:
+        if not config.setting['toolbar_multiselect']:
             directory = QtWidgets.QFileDialog.getExistingDirectory(self, "", current_directory)
             if directory:
                 dir_list.append(directory)
@@ -1288,15 +1291,15 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         dir_count = len(dir_list)
         if dir_count:
             parent = os.path.dirname(dir_list[0]) if dir_count > 1 else dir_list[0]
-            config.persist["current_directory"] = parent
+            config.persist['current_directory'] = parent
             if dir_count > 1:
                 self.set_statusbar_message(
-                    N_("Adding multiple directories from '%(directory)s' ..."),
+                    N_("Adding multiple directories from '%(directory)s' …"),
                     {'directory': parent}
                 )
             else:
                 self.set_statusbar_message(
-                    N_("Adding directory: '%(directory)s' ..."),
+                    N_("Adding directory: '%(directory)s' …"),
                     {'directory': dir_list[0]}
                 )
 
@@ -1353,11 +1356,11 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def save(self):
         """Tell the tagger to save the selected objects."""
         config = get_config()
-        if config.setting["file_save_warning"]:
+        if config.setting['file_save_warning']:
             count = len(self.tagger.get_files_from_objects(self.selected_objects))
             msg = SaveWarningDialog(self, count)
             proceed_with_save, disable_warning = msg.show()
-            config.setting["file_save_warning"] = not disable_warning
+            config.setting['file_save_warning'] = not disable_warning
         else:
             proceed_with_save = True
         if proceed_with_save:
@@ -1401,7 +1404,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 def on_finished(result):
                     callback(config.setting['fingerprinting_system'])
 
-                dialog = self.show_options("fingerprinting")
+                dialog = self.show_options('fingerprinting')
                 dialog.finished.connect(on_finished)
         else:
             callback(config.setting['fingerprinting_system'])
@@ -1434,7 +1437,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if isinstance(obj, Track) and obj.files:
             obj = obj.files[0]
         if not isinstance(obj, File):
-            log.debug('show_more_tracks expected a File, got %r', obj)
+            log.debug("show_more_tracks expected a File, got %r", obj)
             return
         dialog = TrackSearchDialog(self, force_advanced_search=True)
         dialog.show_similar_tracks(obj)
@@ -1443,7 +1446,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def show_more_albums(self):
         obj = self.get_first_obj_with_type(Cluster)
         if not obj:
-            log.debug('show_more_albums expected a Cluster, got %r', obj)
+            log.debug("show_more_albums expected a Cluster, got %r", obj)
             return
         dialog = AlbumSearchDialog(self, force_advanced_search=True)
         dialog.show_similar_albums(obj)
@@ -1528,13 +1531,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             QtWidgets.QMessageBox.StandardButton.Yes)
         if ret == QtWidgets.QMessageBox.StandardButton.Yes:
             config = get_config()
-            config.setting["browser_integration"] = True
+            config.setting['browser_integration'] = True
             self.tagger.update_browser_integration()
             if addrelease.is_enabled():
                 return True
             else:
                 # Something went wrong, let the user configure browser integration manually
-                self.show_options("network")
+                self.show_options('network')
                 return False
         else:
             return False
@@ -1731,12 +1734,12 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def on_mb_login_finished(self, successful, error_msg):
         if successful:
-            log.debug('MusicBrainz authentication finished successfully')
+            log.debug("MusicBrainz authentication finished successfully")
         else:
-            log.info('MusicBrainz authentication failed: %s', error_msg)
+            log.info("MusicBrainz authentication failed: %s", error_msg)
             QtWidgets.QMessageBox.critical(self,
                 _("Authentication failed"),
-                _('Login failed: %s') % error_msg)
+                _("Login failed: %s") % error_msg)
 
     def show_proxy_dialog(self, proxy, authenticator):
         dialog = ProxyDialog(authenticator, proxy, parent=self)
@@ -1779,7 +1782,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         update_level = config.setting['update_level']
         today = datetime.date.today().toordinal()
         do_auto_update_check = check_for_updates and update_check_days > 0 and today >= last_update_check + update_check_days
-        log.debug('%(check_status)s startup check for program updates.  Today: %(today_date)s, Last check: %(last_check)s (Check interval: %(check_interval)s days), Update level: %(update_level)s (%(update_level_name)s)', {
+        log.debug("%(check_status)s startup check for program updates.  Today: %(today_date)s, Last check: %(last_check)s (Check interval: %(check_interval)s days), Update level: %(update_level)s (%(update_level_name)s)", {
             'check_status': 'Initiating' if do_auto_update_check else 'Skipping',
             'today_date': datetime.date.today(),
             'last_check': str(datetime.date.fromordinal(last_update_check)) if last_update_check > 0 else 'never',
@@ -1820,29 +1823,29 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         Checks that there is a settings dictionary for each profile, and that no profiles
         reference a non-existant file naming script.
         """
-        script_id_key = "selected_file_naming_script_id"
+        script_id_key = 'selected_file_naming_script_id'
         config = get_config()
-        naming_scripts = config.setting["file_renaming_scripts"]
+        naming_scripts = config.setting['file_renaming_scripts']
         naming_script_ids = set(naming_scripts.keys())
         profile_settings = deepcopy(config.profiles[SettingConfigSection.SETTINGS_KEY])
         for profile in config.profiles[SettingConfigSection.PROFILES_KEY]:
-            p_id = profile["id"]
+            p_id = profile['id']
             # Add empty settings if none found for a profile
             if p_id not in profile_settings:
                 log.warning(
-                    "No settings dict found for profile '%s' (\"%s\").  Adding empty dict.",
+                    "No settings dict found for profile '%s' ('%s'). Adding empty dict.",
                     p_id,
-                    profile["title"],
+                    profile['title'],
                 )
                 profile_settings[p_id] = {}
             # Remove any invalid naming script ids from profiles
             if script_id_key in profile_settings[p_id]:
                 if profile_settings[p_id][script_id_key] not in naming_script_ids:
                     log.warning(
-                        "Removing invalid naming script id '%s' from profile '%s' (\"%s\")",
+                        "Removing invalid naming script id '%s' from profile '%s' ('%s')",
                         profile_settings[p_id][script_id_key],
                         p_id,
-                        profile["title"],
+                        profile['title'],
                     )
                     profile_settings[p_id][script_id_key] = None
         config.profiles[SettingConfigSection.SETTINGS_KEY] = profile_settings
@@ -1852,8 +1855,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         """
         if self.script_editor_dialog is None or not isinstance(self.script_editor_dialog, ScriptEditorDialog):
             config = get_config()
-            naming_scripts = config.setting["file_renaming_scripts"]
-            selected_script_id = config.setting["selected_file_naming_script_id"]
+            naming_scripts = config.setting['file_renaming_scripts']
+            selected_script_id = config.setting['selected_file_naming_script_id']
         else:
             naming_scripts = self.script_editor_dialog.naming_scripts
             selected_script_id = self.script_editor_dialog.selected_script_id
@@ -1882,7 +1885,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         """
         config = get_config()
         log.debug("Setting naming script to: %s", id)
-        config.setting["selected_file_naming_script_id"] = id
+        config.setting['selected_file_naming_script_id'] = id
         self.make_script_selector_menu()
         if self.script_editor_dialog:
             self.script_editor_dialog.set_selected_script_id(id)
@@ -1890,8 +1893,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def open_file_naming_script_editor(self):
         """Open the file naming script editor / manager in a new window.
         """
-        self.examples = ScriptEditorExamples(tagger=self.tagger)
-        self.script_editor_dialog = ScriptEditorDialog.show_instance(parent=self, examples=self.examples)
+        examples = ScriptEditorExamples(tagger=self.tagger)
+        self.script_editor_dialog = ScriptEditorDialog.show_instance(parent=self, examples=examples)
         self.script_editor_dialog.signal_save.connect(self.script_editor_save)
         self.script_editor_dialog.signal_selection_changed.connect(self.update_selector_from_script_editor)
         self.script_editor_dialog.signal_index_changed.connect(self.script_editor_index_changed)
@@ -1921,22 +1924,23 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def update_script_editor_example_files(self):
         """Update the list of example files for the file naming script editor.
         """
-        if self.examples:
-            self.examples.update_sample_example_files()
+        script_editor_dialog = self.script_editor_dialog
+        if script_editor_dialog and script_editor_dialog.isVisible():
+            script_editor_dialog.examples.update_sample_example_files()
             self.update_script_editor_examples()
 
     def update_script_editor_examples(self):
         """Update the examples for the file naming script editor, using current settings.
         """
-        if self.examples:
+        script_editor_dialog = self.script_editor_dialog
+        if script_editor_dialog and script_editor_dialog.isVisible():
             config = get_config()
             override = {
                 "rename_files": config.setting["rename_files"],
                 "move_files": config.setting["move_files"],
             }
-            self.examples.update_examples(override=override)
-            if self.script_editor_dialog:
-                self.script_editor_dialog.display_examples()
+            script_editor_dialog.examples.update_examples(override=override)
+            script_editor_dialog.display_examples()
 
     def script_editor_index_changed(self):
         """Process "signal_index_changed" signal from the script editor.
@@ -1989,14 +1993,25 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def show_new_user_dialog(self):
         config = get_config()
-        if config.setting["show_new_user_dialog"]:
+        if config.setting['show_new_user_dialog']:
             msg = NewUserDialog(self)
-            config.setting["show_new_user_dialog"] = msg.show()
+            config.setting['show_new_user_dialog'] = msg.show()
 
     def check_for_plugin_update(self):
         config = get_config()
         if config.setting['check_for_plugin_updates']:
-            self.tagger.pluginmanager.check_update(self)
+            self.tagger.pluginmanager.check_update()
+
+    def show_plugin_update_dialog(self, plugin_names):
+        if not plugin_names:
+            return
+
+        msg = PluginUpdatesDialog(self, plugin_names)
+        show_options_page, perform_check = msg.show()
+        config = get_config()
+        config.setting['check_for_plugin_updates'] = perform_check
+        if show_options_page:
+            self.show_plugins_options_page()
 
     def show_plugins_options_page(self):
         self.show_options(page='plugins')
