@@ -4,9 +4,9 @@
 #
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2006-2007 Lukáš Lalinský
-# Copyright (C) 2008, 2019 Philipp Wolfer
+# Copyright (C) 2008, 2019-2021 Philipp Wolfer
 # Copyright (C) 2012-2013 Michael Wiencek
-# Copyright (C) 2013, 2018-2019 Laurent Monin
+# Copyright (C) 2013, 2018-2021 Laurent Monin
 # Copyright (C) 2017 Sambhav Kothari
 #
 # This program is free software; you can redistribute it and/or
@@ -48,15 +48,15 @@ else:
 
 class CDLookupOptionsPage(OptionsPage):
 
-    NAME = "cdlookup"
+    NAME = 'cdlookup'
     TITLE = N_("CD Lookup")
     PARENT = None
     SORT_ORDER = 50
     ACTIVE = True
-    HELP_URL = '/config/options_cdlookup.html'
+    HELP_URL = "/config/options_cdlookup.html"
 
     options = [
-        TextOption("setting", "cd_lookup_device", ",".join(DEFAULT_DRIVES)),
+        TextOption('setting', 'cd_lookup_device', ','.join(DEFAULT_DRIVES)),
     ]
 
     def __init__(self, parent=None):
@@ -69,7 +69,7 @@ class CDLookupOptionsPage(OptionsPage):
 
     def load(self):
         config = get_config()
-        device = config.setting["cd_lookup_device"]
+        device = config.setting['cd_lookup_device']
         if AUTO_DETECT_DRIVES:
             try:
                 self.ui.cd_lookup_device.setCurrentIndex(self._device_list.index(device))
@@ -86,7 +86,7 @@ class CDLookupOptionsPage(OptionsPage):
         else:
             device = self.ui.cd_lookup_device.text()
             device_list = [device]
-        config.setting["cd_lookup_device"] = device
+        config.setting['cd_lookup_device'] = device
         self.tagger.window.update_cd_lookup_drives(device_list)
 
 
