@@ -5,8 +5,8 @@
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2017 Sambhav Kothari
-# Copyright (C) 2018-2021 Laurent Monin
-# Copyright (C) 2018-2021 Philipp Wolfer
+# Copyright (C) 2018-2022, 2024 Philipp Wolfer
+# Copyright (C) 2018-2023 Laurent Monin
 # Copyright (C) 2021 Tche333
 #
 # This program is free software; you can redistribute it and/or
@@ -42,11 +42,11 @@ from PyQt5 import (
     QtCore,
     QtNetwork,
 )
-from PyQt5.QtCore import (
-    QUrl,
-    QUrlQuery,
+from PyQt5.QtCore import QUrl
+from PyQt5.QtNetwork import (
+    QNetworkRequest,
+    QSslError,
 )
-from PyQt5.QtNetwork import QNetworkRequest
 
 from picard import (
     PICARD_APP_NAME,
@@ -63,10 +63,12 @@ from picard.oauth import OAuthManager
 from picard.util import (
     build_qurl,
     bytes2human,
+    encoded_queryargs,
     parse_json,
 )
 from picard.util.xml import parse_xml
 from picard.webservice import ratecontrol
+from picard.webservice.utils import port_from_qurl
 
 
 COUNT_REQUESTS_DELAY_MS = 250
@@ -77,9 +79,7 @@ USER_AGENT_STRING = '%s-%s/%s (%s;%s-%s)' % (PICARD_ORG_NAME, PICARD_APP_NAME,
                                              platform.platform(),
                                              platform.python_implementation(),
                                              platform.python_version())
-CLIENT_STRING = bytes(QUrl.toPercentEncoding('%s %s-%s' % (PICARD_ORG_NAME,
-                                                           PICARD_APP_NAME,
-                                                           PICARD_VERSION_STR))).decode()
+CLIENT_STRING = '%s %s-%s' % (PICARD_ORG_NAME, PICARD_APP_NAME, PICARD_VERSION_STR)
 
 
 DEFAULT_RESPONSE_PARSER_TYPE = "json"
@@ -96,73 +96,108 @@ class UnknownResponseParserError(Exception):
 
 class WSRequest(QNetworkRequest):
     """Represents a single HTTP request."""
+    _access_token = None
+    _high_prio_no_cache = True
+    _mblogin = None
+    _retries = 0
 
-    def __init__(self, method, host, port, path, handler, parse_response_type=None, data=None,
-                 mblogin=False, cacheloadcontrol=None, refresh=False,
-                 queryargs=None, priority=False, important=False,
-                 request_mimetype=None):
+    response_mimetype = None
+    response_parser = None
+
+    def __init__(
+        self,
+        *,
+        method=None,
+        handler=None,
+        parse_response_type=None,
+        data=None,
+        mblogin=False,
+        cacheloadcontrol=None,
+        refresh=False,
+        priority=False,
+        important=False,
+        request_mimetype=None,
+        url=None,
+        queryargs=None,
+        unencoded_queryargs=None,
+    ):
         """
         Args:
             method: HTTP method.  One of ``GET``, ``POST``, ``PUT``, or ``DELETE``.
-            host: Hostname.
-            port: TCP port number (80 or 443).
-            path: Path component.
             handler: Callback which takes a 3-tuple of `(str:document,
-            QNetworkReply:reply, QNetworkReply.Error:error)`.
+                QNetworkReply:reply, QNetworkReply.Error:error)`.
             parse_response_type: Specifies that request either sends or accepts
-            data as ``application/{{response_mimetype}}``.
+                data as ``application/{{response_mimetype}}``.
             data: Data to include with ``PUT`` or ``POST`` requests.
             mblogin: Hints that this request should be tied to a MusicBrainz
             account, requiring that we obtain an OAuth token first.
-            cacheloadcontrol: See `QNetworkRequest.CacheLoadControlAttribute`.
+            cacheloadcontrol: See `QNetworkRequest.Attribute.CacheLoadControlAttribute`.
             refresh: Indicates a user-specified resource refresh, such as when
-            the user wishes to reload a release.  Marks the request as high priority
-            and disables caching.
-            queryargs: `dict` of query arguments.
-            retries: Current retry attempt number.
+                the user wishes to reload a release.  Marks the request as high priority
+                and disables caching.
             priority: Indicates that this is a high priority request.
             important: Indicates that this is an important request.
             request_mimetype: Set the Content-Type header.
+            url: URL passed as a string or as a QUrl to use for this request
+            queryargs: Encoded query arguments, a dictionary mapping field names to values
+            unencoded_queryargs: Unencoded query arguments, a dictionary mapping field names to values
         """
-        url = build_qurl(host, port, path=path, queryargs=queryargs)
+        # mandatory parameters
+        self.method = method
+        if self.method not in {'GET', 'PUT', 'DELETE', 'POST'}:
+            raise AssertionError('invalid method')
+
+        self.handler = handler
+        if self.handler is None:
+            raise AssertionError('handler undefined')
+
+        if url is None:
+            raise AssertionError('URL undefined')
+
+        if not isinstance(url, QUrl):
+            url = QUrl(url)
+
+        if queryargs is not None or unencoded_queryargs is not None:
+            if queryargs is None:
+                queryargs = {}
+            if unencoded_queryargs:
+                queryargs.update(encoded_queryargs(unencoded_queryargs))
+            query = QtCore.QUrlQuery(url)
+            for k, v in queryargs.items():
+                query.addQueryItem(k, str(v))
+            url.setQuery(query)
+
         super().__init__(url)
 
-        # These two are codependent (see _update_authorization_header) and must
-        # be initialized explicitly.
-        self._access_token = None
-        self._mblogin = None
+        # To simulate an ssl error, uncomment following lines
+        # ssl = self.sslConfiguration()
+        # ssl.setCaCertificates(list())
+        # self.setSslConfiguration(ssl)
 
-        self._retries = 0
-
-        self.method = method
-        self.host = host
-        self.port = port
-        self.path = path
-        self.handler = handler
+        # optional parameters
         self.parse_response_type = parse_response_type
-        self.response_parser = None
-        self.response_mimetype = None
         self.request_mimetype = request_mimetype
         self.data = data
         self.mblogin = mblogin
         self.cacheloadcontrol = cacheloadcontrol
         self.refresh = refresh
-        self.queryargs = queryargs
         self.priority = priority
         self.important = important
-        self._high_prio_no_cache = False
 
-        self.access_token = None
-        self._init_headers()
+        # set headers and attributes
+        self.access_token = None  # call _update_authorization_header
 
-    def _init_headers(self):
-        self.setHeader(QNetworkRequest.UserAgentHeader, USER_AGENT_STRING)
+        if self.method == 'GET':
+            self._high_prio_no_cache = self.refresh
+            self.setAttribute(QNetworkRequest.Attribute.HttpPipeliningAllowedAttribute, True)
+
+        self.setHeader(QNetworkRequest.KnownHeaders.UserAgentHeader, USER_AGENT_STRING)
 
         if self.mblogin or self._high_prio_no_cache:
-            self.setPriority(QNetworkRequest.HighPriority)
-            self.setAttribute(QNetworkRequest.CacheLoadControlAttribute, QNetworkRequest.AlwaysNetwork)
+            self.setPriority(QNetworkRequest.Priority.HighPriority)
+            self.setAttribute(QNetworkRequest.Attribute.CacheLoadControlAttribute, QNetworkRequest.CacheLoadControl.AlwaysNetwork)
         elif self.cacheloadcontrol is not None:
-            self.setAttribute(QNetworkRequest.CacheLoadControlAttribute, self.cacheloadcontrol)
+            self.setAttribute(QNetworkRequest.Attribute.CacheLoadControlAttribute, self.cacheloadcontrol)
 
         if self.parse_response_type:
             try:
@@ -175,14 +210,29 @@ class WSRequest(QNetworkRequest):
 
         if self.data:
             if not self.request_mimetype:
-                self.request_mimetype = self.response_mimetype or "application/x-www-form-urlencoded"
-            self.setHeader(QNetworkRequest.ContentTypeHeader, self.request_mimetype)
+                self.request_mimetype = self.response_mimetype or 'application/x-www-form-urlencoded'
+            self.setHeader(QNetworkRequest.KnownHeaders.ContentTypeHeader, self.request_mimetype)
+
+    @property
+    def has_auth(self):
+        return self.mblogin and self.access_token
 
     def _update_authorization_header(self):
-        authorization = b""
-        if self.mblogin and self.access_token:
-            authorization = ("Bearer %s" % self.access_token).encode('utf-8')
-        self.setRawHeader(b"Authorization", authorization)
+        if self.has_auth:
+            auth = 'Bearer ' + self.access_token
+            self.setRawHeader(b'Authorization', auth.encode('utf-8'))
+
+    @property
+    def host(self):
+        return self.url().host()
+
+    @property
+    def port(self):
+        return port_from_qurl(self.url())
+
+    @property
+    def path(self):
+        return self.url().path()
 
     @property
     def access_token(self):
@@ -218,48 +268,6 @@ class WSRequest(QNetworkRequest):
         return self._retries
 
 
-class WSGetRequest(WSRequest):
-
-    def __init__(self, *args, **kwargs):
-        super().__init__("GET", *args, **kwargs)
-
-    def _init_headers(self):
-        self._high_prio_no_cache = self.refresh
-        super()._init_headers()
-        self.setAttribute(QNetworkRequest.HttpPipeliningAllowedAttribute,
-                          True)
-
-
-class WSPutRequest(WSRequest):
-
-    def __init__(self, *args, **kwargs):
-        super().__init__("PUT", *args, **kwargs)
-
-    def _init_headers(self):
-        self._high_prio_no_cache = True
-        super()._init_headers()
-
-
-class WSDeleteRequest(WSRequest):
-
-    def __init__(self, *args, **kwargs):
-        super().__init__("DELETE", *args, **kwargs)
-
-    def _init_headers(self):
-        self._high_prio_no_cache = True
-        super()._init_headers()
-
-
-class WSPostRequest(WSRequest):
-
-    def __init__(self, *args, **kwargs):
-        super().__init__("POST", *args, **kwargs)
-
-    def _init_headers(self):
-        self._high_prio_no_cache = True
-        super()._init_headers()
-
-
 class RequestTask(namedtuple('RequestTask', 'hostkey func priority')):
 
     @staticmethod
@@ -327,6 +335,7 @@ class WebService(QtCore.QObject):
         self.manager = QtNetwork.QNetworkAccessManager()
         self._network_accessible_changed(self.manager.networkAccessible())
         self.manager.networkAccessibleChanged.connect(self._network_accessible_changed)
+        self.manager.sslErrors.connect(self.ssl_errors)
         self.oauth_manager = OAuthManager(self)
         self.set_cache()
         self.setup_proxy()
@@ -334,34 +343,48 @@ class WebService(QtCore.QObject):
         self.set_transfer_timeout(config.setting['network_transfer_timeout_seconds'])
         self.manager.finished.connect(self._process_reply)
         self._request_methods = {
-            "GET": self.manager.get,
-            "POST": self.manager.post,
-            "PUT": self.manager.put,
-            "DELETE": self.manager.deleteResource
+            'GET': self.manager.get,
+            'POST': self.manager.post,
+            'PUT': self.manager.put,
+            'DELETE': self.manager.deleteResource
         }
         self._init_queues()
         self._init_timers()
 
+    def ssl_errors(self, reply, errors):
+        # According to forums, sometimes sslErrors is triggered with errors set to NoError
+        # This can also be used to ignore others if needed
+        ignored_errors = {
+            QSslError.NoError,
+        }
+        has_errors = False
+        for error in errors:
+            if error not in ignored_errors:
+                has_errors = True
+                log.error("SSL error: %s" % error.errorString())
+        if not has_errors:
+            reply.ignoreSslErrors()
+
     @staticmethod
     def http_response_code(reply):
-        response_code = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute)
+        response_code = reply.attribute(QNetworkRequest.Attribute.HttpStatusCodeAttribute)
         return int(response_code) if response_code else 0
 
     @staticmethod
     def http_response_phrase(reply):
-        return reply.attribute(QNetworkRequest.HttpReasonPhraseAttribute)
+        return reply.attribute(QNetworkRequest.Attribute.HttpReasonPhraseAttribute)
 
     @staticmethod
-    def http_response_safe_url(reply):
-        return reply.request().url().toString(QUrl.RemoveUserInfo)
+    def display_url(url):
+        return url.toDisplayString(QUrl.UrlFormattingOption.RemoveUserInfo | QUrl.ComponentFormattingOption.EncodeSpaces)
 
     def _network_accessible_changed(self, accessible):
         # Qt's network accessibility check sometimes fails, e.g. with VPNs on Windows.
         # If the accessibility is reported to be not accessible, set it to
         # unknown instead. Let's just try any request and handle the error.
         # See https://tickets.metabrainz.org/browse/PICARD-1791
-        if accessible == QtNetwork.QNetworkAccessManager.NotAccessible:
-            self.manager.setNetworkAccessible(QtNetwork.QNetworkAccessManager.UnknownAccessibility)
+        if accessible == QtNetwork.QNetworkAccessManager.NetworkAccessibility.NotAccessible:
+            self.manager.setNetworkAccessible(QtNetwork.QNetworkAccessManager.NetworkAccessibility.UnknownAccessibility)
         log.debug("Network accessible requested: %s, actual: %s", accessible, self.manager.networkAccessible())
 
     def _init_queues(self):
@@ -392,17 +415,17 @@ class WebService(QtCore.QObject):
     def setup_proxy(self):
         proxy = QtNetwork.QNetworkProxy()
         config = get_config()
-        if config.setting["use_proxy"]:
-            if config.setting["proxy_type"] == 'socks':
-                proxy.setType(QtNetwork.QNetworkProxy.Socks5Proxy)
+        if config.setting['use_proxy']:
+            if config.setting['proxy_type'] == 'socks':
+                proxy.setType(QtNetwork.QNetworkProxy.ProxyType.Socks5Proxy)
             else:
-                proxy.setType(QtNetwork.QNetworkProxy.HttpProxy)
-            proxy.setHostName(config.setting["proxy_server_host"])
-            proxy.setPort(config.setting["proxy_server_port"])
-            if config.setting["proxy_username"]:
-                proxy.setUser(config.setting["proxy_username"])
-            if config.setting["proxy_password"]:
-                proxy.setPassword(config.setting["proxy_password"])
+                proxy.setType(QtNetwork.QNetworkProxy.ProxyType.HttpProxy)
+            proxy.setHostName(config.setting['proxy_server_host'])
+            proxy.setPort(config.setting['proxy_server_port'])
+            if config.setting['proxy_username']:
+                proxy.setUser(config.setting['proxy_username'])
+            if config.setting['proxy_password']:
+                proxy.setPassword(config.setting['proxy_password'])
         self.manager.setProxy(proxy)
 
     def set_transfer_timeout(self, timeout):
@@ -431,7 +454,7 @@ class WebService(QtCore.QObject):
         # Aborts a request if no data gets transferred for TRANSFER_TIMEOUT milliseconds.
         timer = QtCore.QTimer(self)
         timer.setSingleShot(True)
-        timer.setTimerType(QtCore.Qt.PreciseTimer)
+        timer.setTimerType(QtCore.Qt.TimerType.PreciseTimer)
         timer.timeout.connect(partial(self._timeout_request, reply))
         reply.finished.connect(timer.stop)
         reset_callback = partial(self._reset_transfer_timeout, timer)
@@ -461,42 +484,38 @@ class WebService(QtCore.QObject):
             from QT 4.7
         """
         return leftUrl.port(80) == rightUrl.port(80) and \
-            leftUrl.toString(QUrl.RemovePort) == rightUrl.toString(QUrl.RemovePort)
-
-    @staticmethod
-    def url_port(url):
-        if url.scheme() == 'https':
-            return url.port(443)
-        return url.port(80)
+            leftUrl.toString(QUrl.UrlFormattingOption.RemovePort) == rightUrl.toString(QUrl.UrlFormattingOption.RemovePort)
 
     def _handle_redirect(self, reply, request, redirect):
-        url = request.url()
         error = int(reply.error())
         # merge with base url (to cover the possibility of the URL being relative)
-        redirect = url.resolved(redirect)
-        if not WebService.urls_equivalent(redirect, reply.request().url()):
-            log.debug("Redirect to %s requested", redirect.toString(QUrl.RemoveUserInfo))
-            redirect_host = redirect.host()
-            redirect_port = self.url_port(redirect)
-            redirect_query = dict(QUrlQuery(redirect).queryItems(QUrl.FullyEncoded))
-            redirect_path = redirect.path()
+        redirect_url = request.url().resolved(redirect)
+        reply_url = reply.request().url()
+        display_redirect_url = self.display_url(redirect_url)
+        display_reply_url = self.display_url(reply_url)
+        if not WebService.urls_equivalent(redirect_url, reply_url):
+            log.debug("Redirect to %s requested", display_redirect_url)
 
-            original_host = url.host()
-            original_port = self.url_port(url)
-            original_host_key = (original_host, original_port)
-            redirect_host_key = (redirect_host, redirect_port)
-            ratecontrol.copy_minimal_delay(original_host_key, redirect_host_key)
+            redirect_request = WSRequest(
+                method='GET',
+                url=redirect_url,
+                handler=request.handler,
+                parse_response_type=request.parse_response_type,
+                priority=True,
+                important=True,
+                mblogin=request.mblogin,
+                cacheloadcontrol=request.attribute(QNetworkRequest.Attribute.CacheLoadControlAttribute),
+                refresh=request.refresh,
+            )
 
-            self.get(redirect_host,
-                     redirect_port,
-                     redirect_path,
-                     request.handler, request.parse_response_type, priority=True, important=True,
-                     refresh=request.refresh, queryargs=redirect_query, mblogin=request.mblogin,
-                     cacheloadcontrol=request.attribute(QNetworkRequest.CacheLoadControlAttribute))
+            ratecontrol.copy_minimal_delay(
+                request.get_host_key(),
+                redirect_request.get_host_key(),
+            )
+
+            self.add_request(redirect_request)
         else:
-            log.error("Redirect loop: %s",
-                      self.http_response_safe_url(reply)
-                      )
+            log.error("Redirect loop: %s", display_reply_url)
             request.handler(reply.readAll(), reply, error)
 
     def _handle_reply(self, reply, request):
@@ -510,11 +529,11 @@ class WebService(QtCore.QObject):
         error = int(reply.error())
         handler = request.handler
         response_code = self.http_response_code(reply)
-        url = self.http_response_safe_url(reply)
+        display_reply_url = self.display_url(reply.request().url())
         if error:
             errstr = reply.errorString()
-            log.error("Network request error for %s: %s (QT code %d, HTTP code %d)",
-                      url, errstr, error, response_code)
+            log.error("Network request error for %s -> %s (QT code %d, HTTP code %d)",
+                      display_reply_url, errstr, error, response_code)
             if (not request.max_retries_reached()
                 and (response_code == 503
                      or response_code == 429
@@ -525,7 +544,7 @@ class WebService(QtCore.QObject):
                      )):
                 slow_down = True
                 retries = request.mark_for_retry()
-                log.debug("Retrying %s (#%d)", url, retries)
+                log.debug("Retrying %s (#%d)", display_reply_url, retries)
                 self.add_request(request)
 
             elif handler is not None:
@@ -534,11 +553,11 @@ class WebService(QtCore.QObject):
             slow_down = (slow_down or response_code >= 500)
 
         else:
-            redirect = reply.attribute(QNetworkRequest.RedirectionTargetAttribute)
-            from_cache = reply.attribute(QNetworkRequest.SourceIsFromCacheAttribute)
+            redirect = reply.attribute(QNetworkRequest.Attribute.RedirectionTargetAttribute)
+            from_cache = reply.attribute(QNetworkRequest.Attribute.SourceIsFromCacheAttribute)
             cached = ' (CACHED)' if from_cache else ''
-            log.debug("Received reply for %s: HTTP %d (%s) %s",
-                      url,
+            log.debug("Received reply for %s -> HTTP %d (%s) %s",
+                      display_reply_url,
                       response_code,
                       self.http_response_phrase(reply),
                       cached
@@ -552,13 +571,14 @@ class WebService(QtCore.QObject):
                         document = request.response_parser(reply)
                         log.debug("Response received: %s", document)
                     except Exception as e:
-                        log.error("Unable to parse the response for %s: %s", url, e)
-                        document = reply.readAll()
+                        log.error("Unable to parse the response for %s -> %s", display_reply_url, e)
+                        document = bytes(reply.readAll())
                         error = e
                     finally:
                         handler(document, reply, error)
                 else:
-                    handler(reply.readAll(), reply, error)
+                    # readAll() returns QtCore.QByteArray, so convert to bytes
+                    handler(bytes(reply.readAll()), reply, error)
 
         ratecontrol.adjust(hostkey, slow_down)
 
@@ -566,7 +586,8 @@ class WebService(QtCore.QObject):
         try:
             request = self._active_requests.pop(reply)
         except KeyError:
-            log.error("Request not found for %s", self.http_response_safe_url(reply))
+            display_reply_url = self.display_url(reply.request().url())
+            log.error("Request not found for %s", display_reply_url)
             return
         try:
             self._handle_reply(reply, request)
@@ -577,40 +598,107 @@ class WebService(QtCore.QObject):
     def get(self, host, port, path, handler, parse_response_type=DEFAULT_RESPONSE_PARSER_TYPE,
             priority=False, important=False, mblogin=False, cacheloadcontrol=None, refresh=False,
             queryargs=None):
-        request = WSGetRequest(host, port, path, handler, parse_response_type=parse_response_type,
-                               mblogin=mblogin, cacheloadcontrol=cacheloadcontrol, refresh=refresh,
-                               queryargs=queryargs, priority=priority, important=important)
+        log.warning("This method is deprecated, use WebService.get_url() instead")
+        request = WSRequest(
+            method='GET',
+            url=build_qurl(host, port, path=path, queryargs=queryargs),
+            handler=handler,
+            parse_response_type=parse_response_type,
+            priority=priority,
+            important=important,
+            mblogin=mblogin,
+            cacheloadcontrol=cacheloadcontrol,
+            refresh=refresh,
+        )
         return self.add_request(request)
 
     def post(self, host, port, path, data, handler, parse_response_type=DEFAULT_RESPONSE_PARSER_TYPE,
              priority=False, important=False, mblogin=True, queryargs=None, request_mimetype=None):
-        request = WSPostRequest(host, port, path, handler, parse_response_type=parse_response_type,
-                                data=data, mblogin=mblogin, queryargs=queryargs,
-                                priority=priority, important=important,
-                                request_mimetype=request_mimetype)
+        log.warning("This method is deprecated, use WebService.post_url() instead")
+        request = WSRequest(
+            method='POST',
+            url=build_qurl(host, port, path=path, queryargs=queryargs),
+            handler=handler,
+            parse_response_type=parse_response_type,
+            priority=priority,
+            important=important,
+            mblogin=mblogin,
+            data=data,
+            request_mimetype=request_mimetype,
+        )
         log.debug("POST-DATA %r", data)
         return self.add_request(request)
 
     def put(self, host, port, path, data, handler, priority=True, important=False, mblogin=True,
             queryargs=None, request_mimetype=None):
-        request = WSPutRequest(host, port, path, handler, data=data, mblogin=mblogin,
-                               queryargs=queryargs, priority=priority,
-                               important=important, request_mimetype=request_mimetype)
+        log.warning("This method is deprecated, use WebService.put_url() instead")
+        request = WSRequest(
+            method='PUT',
+            url=build_qurl(host, port, path=path, queryargs=queryargs),
+            handler=handler,
+            priority=priority,
+            important=important,
+            mblogin=mblogin,
+            data=data,
+            request_mimetype=request_mimetype,
+        )
         return self.add_request(request)
 
     def delete(self, host, port, path, handler, priority=True, important=False, mblogin=True,
                queryargs=None):
-        request = WSDeleteRequest(host, port, path, handler, mblogin=mblogin,
-                                  queryargs=queryargs, priority=priority, important=important)
+        log.warning("This method is deprecated, use WebService.delete_url() instead")
+        request = WSRequest(
+            method='DELETE',
+            url=build_qurl(host, port, path=path, queryargs=queryargs),
+            handler=handler,
+            priority=priority,
+            important=important,
+            mblogin=mblogin,
+        )
         return self.add_request(request)
 
     def download(self, host, port, path, handler, priority=False,
                  important=False, cacheloadcontrol=None, refresh=False,
                  queryargs=None):
-        return self.get(host, port, path, handler, parse_response_type=None,
-                        priority=priority, important=important,
-                        cacheloadcontrol=cacheloadcontrol, refresh=refresh,
-                        queryargs=queryargs)
+        log.warning("This method is deprecated, use WebService.download_url() instead")
+        request = WSRequest(
+            method='GET',
+            url=build_qurl(host, port, path=path, queryargs=queryargs),
+            handler=handler,
+            priority=priority,
+            important=important,
+            cacheloadcontrol=cacheloadcontrol,
+            refresh=refresh,
+        )
+        return self.add_request(request)
+
+    def get_url(self, **kwargs):
+        kwargs['method'] = 'GET'
+        kwargs['parse_response_type'] = kwargs.get('parse_response_type', DEFAULT_RESPONSE_PARSER_TYPE)
+        return self.add_request(WSRequest(**kwargs))
+
+    def post_url(self, **kwargs):
+        kwargs['method'] = 'POST'
+        kwargs['parse_response_type'] = kwargs.get('parse_response_type', DEFAULT_RESPONSE_PARSER_TYPE)
+        kwargs['mblogin'] = kwargs.get('mblogin', True)
+        log.debug("POST-DATA %r", kwargs['data'])
+        return self.add_request(WSRequest(**kwargs))
+
+    def put_url(self, **kwargs):
+        kwargs['method'] = 'PUT'
+        kwargs['priority'] = kwargs.get('priority', True)
+        kwargs['mblogin'] = kwargs.get('mblogin', True)
+        return self.add_request(WSRequest(**kwargs))
+
+    def delete_url(self, **kwargs):
+        kwargs['method'] = 'DELETE'
+        kwargs['priority'] = kwargs.get('priority', True)
+        kwargs['mblogin'] = kwargs.get('mblogin', True)
+        return self.add_request(WSRequest(**kwargs))
+
+    def download_url(self, **kwargs):
+        kwargs['method'] = 'GET'
+        return self.add_request(WSRequest(**kwargs))
 
     def stop(self):
         for reply in list(self._active_requests):
